/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_VIDEO_IOCTL_H__
#define __CX88_VIDEO_IOCTL_H__

#define CX88_VIDEO_MAX_I2C_DEVICES 4
#define CX88_VIDEO_MAX_DEVNODE_NAME 32

typedef struct
{   
   uint8_t in_i2c_addrs[CX88_VIDEO_MAX_I2C_DEVICES]; /* Addresses of I2C devices */
   
   /* Returns names of I2C device files */
   char out_i2c_devnodes[CX88_VIDEO_MAX_I2C_DEVICES][CX88_VIDEO_MAX_DEVNODE_NAME];
   
} cx88_video_config;

typedef struct
{
   uint16_t scale_value;
   
   #define CX88_VIDEO_SCALE_NONE    0
   #define CX88_VIDEO_SCALE_PERCENT 1
   #define CX88_VIDEO_SCALE_PIXELS  2 
   uint8_t scale_type;
} cx88_video_scale;

typedef struct
{
   uint16_t crop_offset;
   uint16_t crop_value;
      
   #define CX88_VIDEO_CROP_NONE     0
   #define CX88_VIDEO_CROP_PERCENT  1
   #define CX88_VIDEO_CROP_PIXELS   2 
   uint8_t crop_type;
} cx88_video_crop;

typedef struct
{
   uint8_t enable;
   
   #define CX88_VIDEO_SWAP_WORDS (1 << 0)
   #define CX88_VIDEO_SWAP_BYTES (1 << 1)
   uint8_t swap;

   #define CX88_VIDEO_RGB32      0
   #define CX88_VIDEO_RGB24      1
   #define CX88_VIDEO_RGB16      2
   #define CX88_VIDE0_RGB15      3
   #define CX88_VIDEO_YUV422     4
   #define CX88_VIDEO_BTYUV411   5
   #define CX88_VIDEO_Y8         6
   #define CX88_VIDEO_TGB8       7
   #define CX88_VIDEO_YCC422     8
   #define CX88_VIDEO_YCC411     9
   #define CX88_VIDEO_RAW        14
   uint8_t color_format;
} cx88_video_field;

typedef struct
{
   #define CX88_VIDEO_FMT_AUTO         0
   #define CX88_VIDEO_FMT_NTSCM        1
   #define CX88_VIDEO_FMT_NTSCJ        2
   #define CX88_VIDEO_FMT_NTSC443      3
   #define CX88_VIDEO_FMT_PALBDGHI     4
   #define CX88_VIDEO_FMT_PALM         5
   #define CX88_VIDEO_FMT_PALN         6
   #define CX88_VIDEO_FMT_PALNC        7
   #define CX88_VIDEO_FMT_PAL60        8
   #define CX88_VIDEO_FMT_SECAM        9
   uint8_t in_video_format;
   
   cx88_video_scale in_hscale;
   /* Horizontal cropping is applied AFTER horizontal scaling */
   cx88_video_crop  in_hcrop;
   
   /* Vertical cropping is applied BEFORE vertical scaling */
   cx88_video_crop  in_vcrop;
   cx88_video_scale in_vscale;
   
   cx88_video_field in_even_field;
   cx88_video_field in_odd_field;
   
   /* Number of frames to allocate */
   uint8_t in_num_frames;
   
   /* Buffer configuration */
   union
   {
      struct
      {
         /* mmap() works at page granularity, so to acommodate page-rounding, 
          * applications will need to use out_mmap_size to determine the total number
          * of bytes to mmap(), and out_buffer_size to compute buffer offsets within
          * the mmap'ed region. */
         uint32_t out_buffer_size; 
         uint32_t out_mmap_size;
      } out_kernelbuf; /* Kernel-mode buffer allocation */
      void *in_userbuf; /* User-mode buffer allocation */
   } bufconfig;
      
   #define CX88_VIDEO_BUF_KERNEL 0
   #define CX88_VIDEO_BUF_USER 1
   uint8_t in_buftype; /* Select user- or kernel-mode buffers */
   
} cx88_video_format;

typedef struct
{   
   #define CX88_VIDEO_INPUT_MUX0             0
   #define CX88_VIDEO_INPUT_MUX1             1
   #define CX88_VIDEO_INPUT_MUX2             2
   #define CX88_VIDEO_INPUT_MUX3             3
   #define CX88_VIDEO_INPUT_COMPOSITE_VIDEO (0 << 2)
   #define CX88_VIDEO_INPUT_COMPOSITE_TV    (1 << 2)
   #define CX88_VIDEO_INPUT_SVIDEO          (2 << 2)
   #define CX88_VIDEO_INPUT_VIP             (3 << 2)
   uint8_t in_input_source;
   
} cx88_video_capture;

typedef struct
{
   int8_t in_hue;
   int8_t in_brightness;
   uint8_t in_contrast;
   uint8_t in_saturation_u;
   uint8_t in_saturation_v;
   uint8_t in_drop_percentage;
} cx88_video_adjust;

#endif
